from django import forms
from django.core.validators import RegexValidator, MinLengthValidator, MaxLengthValidator
from django.utils import timezone
from datetime import date, timedelta
import re
from .models import User

class UserRegistrationForm(forms.ModelForm):
    # Password field with validation
    password = forms.CharField(
        widget=forms.PasswordInput(),
        validators=[
            RegexValidator(
                regex=r'^(?=.*[A-Za-z])(?=.*\d)(?=.*[@$!%*#?&._-])[A-Za-z\d@$!%*#?&._-]{8,}$',
                message='Password harus mengandung minimal satu huruf, satu angka, dan satu karakter spesial dengan panjang minimal 8'
            )
        ]
    )
    
    class Meta:
        model = User
        fields = ['name', 'password', 'date_of_birth', 
                 'phone_number', 'email', 'blog_url', 'self_description', 
                 'entry_year', 'study_program_code']
        widgets = {
            'date_of_birth': forms.DateInput(attrs={'type': 'date'}),
            'self_description': forms.Textarea(attrs={'rows': 4}),
        }
    
    def clean_name(self):
        # Validasi nama - memperbolehkan huruf, angka, spasi, dan karakter khusus tertentu
        name = self.cleaned_data.get('name')
        
        # Periksa apakah nama hanya mengandung huruf, angka, spasi, dan karakter yang diizinkan
        allowed_chars = r'[a-zA-Z0-9 ._-]+'
        if not re.match(f'^{allowed_chars}$', name):
            raise forms.ValidationError('Nama hanya boleh berisi huruf, angka, spasi, dan karakter (., _, -)')
        
        return name
    
    def clean_phone_number(self):
        # Sanitasi nomor HP - hapus tanda + dan -
        phone = self.cleaned_data.get('phone_number')
        if phone:
            # Hapus karakter + dan -
            phone = phone.replace('+', '')
            phone = phone.replace('-', '')
            
            # Periksa apakah nomor telepon hanya berisi angka
            if not phone.isdigit():
                raise forms.ValidationError('Nomor HP hanya boleh berisi angka')
            
            # Periksa panjang nomor HP
            if len(phone) < 8 or len(phone) > 15:
                raise forms.ValidationError('Nomor HP harus memiliki panjang antara 8 sampai 15 digit')
                
        return phone
    
    def clean_date_of_birth(self):
        dob = self.cleaned_data.get('date_of_birth')
        if dob:
            # Check if user is at least 12 years old
            min_age = 12
            today = date.today()
            min_date = today.replace(year=today.year - min_age)
            
            if dob > min_date:
                raise forms.ValidationError(f'Usia pengguna minimal {min_age} tahun')
        return dob