from django.db import models
from django.contrib.auth.models import AbstractUser
from django.core.validators import RegexValidator, MinLengthValidator, MaxLengthValidator, MinValueValidator, MaxValueValidator
from django.utils import timezone
import datetime
from datetime import date

class User(AbstractUser):
    name = models.CharField(
        max_length=255,
        validators=[
            RegexValidator(
                regex=r'^[a-zA-Z0-9 ._-]+$',
                message='Nama hanya boleh berisi huruf, angka, spasi, dan karakter (., _, -)'
            )
        ]
    )
    
    # Date of birth validation - minimum age of 12 years
    date_of_birth = models.DateField()
    
    # Phone number validation - format and length constraints
    # Hanya berisi angka, panjang 8-15
    phone_number = models.CharField(
        max_length=15,
        validators=[
            RegexValidator(
                regex=r'^[0-9]+$',
                message='Nomor HP hanya boleh berisi angka'
            ),
            MinLengthValidator(8, 'Nomor HP minimal harus 8 karakter'),
            MaxLengthValidator(15, 'Nomor HP tidak boleh lebih dari 15 karakter')
        ]
    )
    
    # Email validation (Django's built-in validation)
    email = models.EmailField(max_length=255)
    
    # URL validation
    blog_url = models.URLField(max_length=255)
    
    # Self description validation - min and max length
    self_description = models.TextField(
        validators=[
            MinLengthValidator(5, 'Deskripsi diri minimal 5 karakter'),
            MaxLengthValidator(1000, 'Deskripsi diri maksimal 1000 karakter')
        ]
    )
    
    # Additional variables
    entry_year = models.IntegerField(
        validators=[
            MinValueValidator(1900, 'Tahun minimal adalah 1900'),
            MaxValueValidator(date.today().year, f'Tahun tidak boleh lebih dari {date.today().year}')
        ]
    )
    
    study_program_code = models.CharField(
        max_length=8,
        validators=[
            RegexValidator(
                regex=r'^[A-Z]{3}[0-9]{5}$',
                message='Kode program studi harus dimulai dengan 3 huruf kapital diikuti oleh 5 digit'
            )
        ]
    )
    
    def clean(self):
        # Custom validation for minimum age
        if self.date_of_birth:
            today = timezone.now().date()
            min_age = 12
            min_date = today.replace(year=today.year - min_age)
            
            if self.date_of_birth > min_date:
                from django.core.exceptions import ValidationError
                raise ValidationError({'date_of_birth': f'Usia pengguna minimal {min_age} tahun'})
        
        super().clean()